import { useState } from 'react';
import { useQuery, useMutation } from '@apollo/client';
import { GET_GROUPS, GET_TASKS, GET_STEP, GET_RECENT_TASKS, GET_FREQUENT_TASKS } from '../graphql/queries';
import { PRINT_TASK, PRINT_STEP, CREATE_NOTE, CREATE_GROUP, CREATE_TASK, CREATE_STEP } from '../graphql/mutations';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';
import { doesExist, isArray } from '../utils/typeGuards';

interface GraphQLStep {
  id: number;
  name: string;
  instructions: string;
  taskId: number;
  order: number;
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLNote {
  id: number;
  content: string;
  created_at: string;
  updated_at: string;
  user: {
    id: number;
    name: string;
  };
}

interface GraphQLTask {
  id: number;
  name: string;
  groupId: number;
  steps: GraphQLStep[];
  notes: GraphQLNote[];
  print_count: number;
  last_printed_at: string | null;
}

interface GraphQLGroup {
  id: number;
  name: string;
  tasks: GraphQLTask[];
  created_at: string;
  updated_at: string;
}

function toStepWithNotes(step: GraphQLStep | null | undefined): StepWithNotes | null {
  if (!doesExist(step)) return null;

  return {
    id: step.id,
    name: step.name,
    instructions: step.instructions,
    task_id: step.taskId,
    order: step.order,
    print_count: step.print_count,
    last_printed_at: step.last_printed_at ? new Date(step.last_printed_at) : undefined,
    notes: isArray(step.notes) ? step.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

function toTaskWithSteps(task: GraphQLTask | null | undefined): TaskWithSteps | null {
  if (!doesExist(task)) return null;

  return {
    id: task.id,
    name: task.name,
    group_id: task.groupId,
    print_count: task.print_count,
    last_printed_at: task.last_printed_at ? new Date(task.last_printed_at) : undefined,
    steps: isArray(task.steps) ? task.steps.map(step => toStepWithNotes(step)).filter(doesExist) : [],
    notes: isArray(task.notes) ? task.notes.map(note => ({
      id: note.id,
      content: note.content,
      created_at: new Date(note.created_at),
      updated_at: new Date(note.updated_at),
      created_by: note.user.id,
      user: {
        id: note.user.id,
        name: note.user.name,
      },
    })) : [],
  };
}

function toGroupWithTasks(group: GraphQLGroup | null | undefined): GroupWithTasks | null {
  if (!doesExist(group)) return null;

  return {
    id: group.id,
    name: group.name,
    created_at: new Date(group.created_at),
    updated_at: new Date(group.updated_at),
    tasks: isArray(group.tasks) ? group.tasks.map(task => toTaskWithSteps(task)).filter(doesExist) : [],
  };
}

export function useTaskData() {
  const [selectedGroup, setSelectedGroup] = useState<GroupWithTasks | undefined>();
  const [selectedTask, setSelectedTask] = useState<TaskWithSteps | undefined>();
  const [selectedStep, setSelectedStep] = useState<StepWithNotes | undefined>();

  const { data: groupsData, loading: groupsLoading, refetch: refetchGroups } = useQuery(GET_GROUPS);
  const { data: tasksData, loading: tasksLoading, refetch: refetchTasks } = useQuery(GET_TASKS, {
    variables: { groupId: selectedGroup?.id },
    skip: !selectedGroup?.id,
  });
  const { data: stepData, loading: stepLoading, refetch: refetchStep } = useQuery(GET_STEP, {
    variables: { id: selectedStep?.id },
    skip: !selectedStep?.id,
  });
  const { data: recentTasksData, loading: recentTasksLoading } = useQuery(GET_RECENT_TASKS);
  const { data: frequentTasksData, loading: frequentTasksLoading } = useQuery(GET_FREQUENT_TASKS);

  const [printTask] = useMutation(PRINT_TASK);
  const [printStep] = useMutation(PRINT_STEP);
  const [createNote] = useMutation(CREATE_NOTE);
  const [createGroup] = useMutation(CREATE_GROUP);
  const [createTask] = useMutation(CREATE_TASK);
  const [createStep] = useMutation(CREATE_STEP);

  const handlePrintTask = async (taskId: string, userId: string) => {
    if (!taskId || !userId) {
      console.error('Cannot print task: missing taskId or userId');
      return;
    }

    try {
      await printTask({ variables: { id: taskId, userId } });
    } catch (error) {
      console.error('Error printing task:', error);
    }
  };

  const handlePrintStep = async (stepId: string, userId: string) => {
    if (!stepId || !userId) {
      console.error('Cannot print step: missing stepId or userId');
      return;
    }

    try {
      await printStep({ variables: { id: stepId, userId } });
    } catch (error) {
      console.error('Error printing step:', error);
    }
  };

  const handleAddNote = async (content: string) => {
    if (!selectedStep?.id) {
      console.error('Cannot add note: missing stepId');
      return;
    }

    try {
      await createNote({
        variables: {
          content,
          stepId: String(selectedStep.id),
        },
      });
      await refetchStep({ id: String(selectedStep.id) });
    } catch (error) {
      console.error('Error adding note:', error);
    }
  };

  const handleCreateGroup = async (name: string) => {
    if (!name) {
      console.error('Cannot create group: missing name');
      return;
    }

    try {
      await createGroup({ variables: { name } });
      await refetchGroups();
    } catch (error) {
      console.error('Error creating group:', error);
    }
  };

  const handleCreateTask = async (name: string, groupId: string) => {
    if (!name || !groupId) {
      console.error('Cannot create task: missing name or groupId');
      return;
    }

    try {
      const variables = {
        name,
        groupId,
      };
      console.log('Creating task with variables:', variables);
      await createTask({ variables });
      await refetchTasks({ groupId });
    } catch (error) {
      console.error('Error creating task:', error);
    }
  };

  const handleCreateStep = async (name: string, instructions: string, taskId: string, order: number) => {
    if (!name || !instructions || !taskId || order < 1) {
      console.error('Cannot create step: missing required fields');
      return;
    }

    try {
      await createStep({ variables: { name, instructions, taskId, order } });
      await refetchStep({ id: taskId });
    } catch (error) {
      console.error('Error creating step:', error);
    }
  };

  const groups = isArray(groupsData?.groups)
    ? groupsData.groups.map((group: GraphQLGroup) => toGroupWithTasks(group)).filter(doesExist)
    : [];

  const recentTasks = isArray(recentTasksData?.recentTasks)
    ? recentTasksData.recentTasks.map((task: GraphQLTask) => toTaskWithSteps(task)).filter(doesExist)
    : [];

  const frequentTasks = isArray(frequentTasksData?.frequentTasks)
    ? frequentTasksData.frequentTasks.map((task: GraphQLTask) => toTaskWithSteps(task)).filter(doesExist)
    : [];

  // Create virtual groups for recent and frequent tasks
  const virtualGroups: GroupWithTasks[] = [
    {
      id: -1, // Use negative IDs for virtual groups
      name: 'Recent Tasks',
      created_at: new Date(),
      updated_at: new Date(),
      tasks: recentTasks,
    },
    {
      id: -2,
      name: 'Frequent Tasks',
      created_at: new Date(),
      updated_at: new Date(),
      tasks: frequentTasks,
    },
  ];

  // Combine virtual groups with regular groups
  const allGroups = [...virtualGroups, ...groups];

  const tasks = isArray(tasksData?.tasks)
    ? tasksData.tasks.map((task: GraphQLTask) => toTaskWithSteps(task)).filter(doesExist)
    : [];

  const step = stepData?.step ? toStepWithNotes(stepData.step) : undefined;

  return {
    groups: allGroups,
    tasks,
    step,
    loading: groupsLoading || tasksLoading || stepLoading || recentTasksLoading || frequentTasksLoading,
    selectedGroup,
    selectedTask,
    selectedStep,
    setSelectedGroup,
    setSelectedTask,
    setSelectedStep,
    handlePrintTask,
    handlePrintStep,
    handleAddNote,
    handleCreateGroup,
    handleCreateTask,
    handleCreateStep,
  };
}