import { Box, Paper, Typography, IconButton, TextField, Button } from '@mui/material';
import ArrowBackIcon from '@mui/icons-material/ArrowBack';
import PrintIcon from '@mui/icons-material/Print';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';
import { CreateButtons } from '../components/CreateButtons';

function isGroupView(selectedGroup?: GroupWithTasks) {
  return !selectedGroup;
}

function isTaskView(selectedGroup?: GroupWithTasks, selectedTask?: TaskWithSteps) {
  return selectedGroup && !selectedTask;
}

function isStepView(selectedTask?: TaskWithSteps, selectedStep?: StepWithNotes) {
  return selectedTask && !selectedStep;
}

export function MobileLayout({
  groups = [],
  selectedGroup,
  selectedTask,
  selectedStep,
  onGroupSelect,
  onTaskSelect,
  onStepSelect,
  onBack,
  onPrintStep,
  onAddNote
}: {
  groups: GroupWithTasks[];
  selectedGroup?: GroupWithTasks;
  selectedTask?: TaskWithSteps;
  selectedStep?: StepWithNotes;
  onGroupSelect: (group: GroupWithTasks | undefined) => void;
  onTaskSelect: (task: TaskWithSteps | undefined) => void;
  onStepSelect: (step: StepWithNotes | undefined) => void;
  onBack: () => void;
  onPrintStep?: (stepId: string, userId: string) => void;
  onAddNote?: (content: string) => void;
}) {
  const groupList: GroupWithTasks[] = groups;
  const taskList: TaskWithSteps[] = (selectedGroup?.tasks as TaskWithSteps[]) ?? [];
  const stepList: StepWithNotes[] = (selectedTask?.steps as StepWithNotes[]) ?? [];

  return (
    <Box sx={{ height: '100vh', p: 2 }}>
      {/* Header with back button */}
      {selectedGroup && !selectedTask && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedGroup.name}</Typography>
        </Box>
      )}

      {selectedTask && !selectedStep && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedTask.name}</Typography>
        </Box>
      )}

      {selectedStep && (
        <Box sx={{ display: 'flex', alignItems: 'center', mb: 2 }}>
          <IconButton onClick={onBack} sx={{ mr: 1 }}>
            <ArrowBackIcon />
          </IconButton>
          <Typography variant="h6">{selectedStep.name}</Typography>
          {onPrintStep && (
            <IconButton
              onClick={() => onPrintStep(String(selectedStep.id), '1')} // TODO: Get real user ID
              sx={{ ml: 'auto' }}
            >
              <PrintIcon />
            </IconButton>
          )}
        </Box>
      )}

      {/* Content */}
      <Paper sx={{ p: 2, overflow: 'auto', height: 'calc(100vh - 80px)' }}>
        {isGroupView(selectedGroup) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Groups</Typography>
              <CreateButtons type="group" />
            </Box>

            {/* Groups List */}
            {groupList.map((group) => (
              <Box
                key={group.id}
                sx={{
                  p: 2,
                  cursor: 'pointer',
                  bgcolor: selectedGroup?.id === group.id ? 'action.selected' : 'transparent',
                  '&:hover': { bgcolor: 'action.hover' },
                }}
                onClick={() => onGroupSelect(group)}
              >
                <Typography>{group.name}</Typography>
              </Box>
            ))}
          </>
        )}

        {isTaskView(selectedGroup, selectedTask) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Tasks</Typography>
              {selectedGroup.id > 0 && <CreateButtons type="task" parentId={String(selectedGroup.id)} />}
            </Box>
            <Box>
              {taskList.map((task) => (
                <Box
                  key={task.id}
                  sx={{
                    p: 2,
                    cursor: 'pointer',
                    bgcolor: selectedTask?.id === task.id ? 'action.selected' : 'transparent',
                  }}
                  onClick={() => onTaskSelect(task)}
                >
                  <Typography>{task.name}</Typography>
                </Box>
              ))}
            </Box>
          </>
        )}

        {isStepView(selectedTask, selectedStep) && (
          <>
            <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
              <Typography variant="h6">Steps</Typography>
              <CreateButtons type="step" parentId={String(selectedTask?.id)} />
            </Box>
            <Box>
              {stepList.map((step) => (
                <Box
                  key={step.id}
                  sx={{
                    p: 2,
                    cursor: 'pointer',
                    bgcolor: selectedStep?.id === step.id ? 'action.selected' : 'transparent',
                  }}
                  onClick={() => onStepSelect(step)}
                >
                  <Typography>{step.name}</Typography>
                </Box>
              ))}
            </Box>
          </>
        )}

        {selectedStep && (
          <>
            <Typography sx={{ mb: 4 }}>{selectedStep.instructions}</Typography>

            <Box sx={{ mb: 4 }}>
              <Typography variant="h6" sx={{ mb: 2 }}>
                Notes ({selectedStep.notes.length})
              </Typography>
              {selectedStep.notes.map((note) => (
                <Box key={note.id} sx={{ mb: 2 }}>
                  <Typography variant="subtitle2">{note.user?.name}</Typography>
                  <Typography>{note.content}</Typography>
                </Box>
              ))}
            </Box>

            <Box sx={{ mb: 4 }}>
              <TextField
                fullWidth
                multiline
                rows={4}
                placeholder="Add a note..."
                sx={{ mb: 2 }}
              />
              {onAddNote && (
                <Button variant="contained" onClick={() => onAddNote('New note')}>
                  Add Note
                </Button>
              )}
            </Box>
          </>
        )}
      </Paper>
    </Box>
  );
}